clear all
close all

%% Load the map and create an occupancy grid
% Load the image
image = imread('../maps/turtlebot-map.pgm');
image = imrotate(image,90);
image = imtranslate(image,[0.8/0.05 0]);

% Transform the image to a binary map
imageNorm = double(image)/255;
imageOccupancy = 1 - imageNorm;

% Create the occupancy map object
occGrid = occupancyMap(imageOccupancy,1/0.05);

% Set the parameters according to the yaml file
occGrid.LocalOriginInWorld = [-10,-10];
occGrid.OccupiedThreshold = 0.65;
occGrid.FreeThreshold = 0.196;

% Show the map
show(occGrid)

%% Plan the path
map = copy(occGrid);
inflate(map,0.1);

bounds = [map.XWorldLimits;...
    map.YWorldLimits; [-pi pi]]; % Bounds on configuration variables
space = stateSpaceSE2(bounds);

stateValidator = validatorOccupancyMap(space); 
stateValidator.Map = map;
stateValidator.ValidationDistance = 0.05; % Distance for path discretization

planner = plannerRRTStar(space, stateValidator);
planner.BallRadiusConstant = 100;
planner.ContinueAfterGoalReached = true;
planner.MaxIterations = 5e5;
planner.MaxNumTreeNodes = 1e5;
planner.MaxConnectionDistance = 0.5;
planner.GoalReachedFcn = @isGoalReached;

start = [-1.0, -2.0, 0.0];
goal = [0.5, 0.5, 0.0];
[pthObj, solnInfo] = plan(planner, start, goal);

%% Plot the planned path
show(occGrid),hold on
plot(solnInfo.TreeData(:,1), solnInfo.TreeData(:,2), '.-')

plot(pthObj.States(:,1), pthObj.States(:,2), 'r-', 'LineWidth', 2),hold off

%% Generate a trajectory with constant linear and angular velocity with respect to the odom frame
v = 1.0;
w = 0.1;
R = rotz(-90);

desired_traj = zeros(size(pthObj.States,1),4);
desired_traj(1,1:2) = R(1:2,1:2)*pthObj.States(1,1:2)';
desired_traj(1,3) = -pi/2+pthObj.States(1,3);
for k=2:size(desired_traj,1)
    desired_traj(k,1:2) = R(1:2,1:2)*pthObj.States(k,1:2)';
    desired_traj(k,3) = -pi/2+pthObj.States(k,3);
    desired_traj(k,4) = desired_traj(k-1,4)+...
        sqrt((desired_traj(k,1)-desired_traj(k-1,1))^2+...
        (desired_traj(k,2)-desired_traj(k-1,2))^2)/v+...
        abs(desired_traj(k,3)-desired_traj(k-1,3))/w;
end

%% Functions
function isReached = isGoalReached(planner, goalState, newState)
    isReached = false;
    threshold = 0.1;
    if planner.StateSpace.distance(newState, goalState) < threshold
        isReached = true;
    end
end